#include "lpc23xx.h"
#include "LED.h"

void UNDEF_Routine();
void SWI_Routine();
void IRQ_Routine();
void FIQ_Routine();

#define FCPU 80 * 1000 * 1000
#define BAUD 115200

/*! Initializes the PLL to run with external 20MHz crystal
	and multiplies it to 80MHz Core-clock
	- original code by Steffen Graf -
*/
void PLL_init(void){
    SCS |= (1<<5);			//Enable main OSC
    while(!(SCS&0x40));	//Wait until it's stable
    CLKSRCSEL = 0x1; 		//Select main OSC as clock source
	
	PLLCON = 0;				//Disable PLL
	PLLFEED = 0xAA;			//send 0xAA55 to apply changes
	PLLFEED = 0x55;
	
	PLLCFG = (1<<16)|11;	//Set PLL setting: div = 1 | mul = 12 | F_PLL = 240 MHz
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	
	PLLCON = 1;				//Enable PLL
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	
	while(!(PLLSTAT&(1<<26)));	//Wait until it is locked
	CCLKCFG   = 2;			//CClk Divider = 3 | CCLK = 80 MHz
	USBCLKCFG = 9;			//USBCLK Divide = 10 | USBCLk = 48 MHz

	PLLCON = 3;				//Set the PLL as clock source
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	
	PCLKSEL0 = 0x55515555;	//peripherals clock for all periperials = CCLK
	PCLKSEL1 = 0x15555555;
	
	MAMCR = 0x00;			//Memory accelerator module off
	MAMTIM = 0x04;			//MAM: 4cycles
	MAMCR = 0x02;			//Enable MAM
}


/*! Set's up UART vor 8n1
*/
void InitUART()
{
	// Power on UART0
	PCONP |= (1 << 3) ;
	// Select CLK (CCLK/4)
	PCLKSEL0 |= (0 << 6) ;
	// select BAUDRATE
	U0LCR |= (1 << 7) ;
	
	U0FDR = (0 << 0) | (1 << 4) ; // UART base clock = clock 
	
	unsigned int baudrate = (FCPU / 16) / BAUD ;
	
	U0DLL = (baudrate & 0xFF) ;
	U0DLM = baudrate >> 8 ;
	U0FCR |= 1 ;
	// activate PIN TXD0,RXD0
	PINSEL0 = (PINSEL0 & (~0xF0)) | (1 << 4) | (1 << 6) ;
	U0LCR = (3<<0);
}

/* send single char over UART */
void UART_sendc(char c)
{
	while ((U0LSR & (1 << 5))==0) ;
	U0THR = c ;
}

/* send string over UART */
void UART_sends(char *s)
{
	while (*s)
	{
		UART_sendc(*s) ;
		s++ ;
	}
}

/* send an unsigned long as hex string over UART */
void UART_sendx(unsigned long val)
{
	UART_sends("0x0") ;
	int i ;
	for (i=8;i>0;i--)
	{
		unsigned char stelle = (val >> ((i-1)*4)) & 0x0F ;
		if (stelle>=10) 
			UART_sendc(stelle - 10 + 'A') ;
		else
			UART_sendc(stelle + '0') ;
		
	}
}

/* send an unsigned long as decimal string over UART */
void UART_sendu32(unsigned long val)
{
	char buffer[10] ;
	int i;
	for (i=0;i<10;i++) buffer[i] = ' ' ;
	buffer[9] = 0 ;
	buffer[8] = '0' ;
	int stelle = 8 ;
	if (!val) 
	{
		UART_sends("0") ;
		return ;
	}
	while (val)
	{
		buffer[stelle] = (val % 10) + '0' ;
		stelle-- ;
		val /= 10 ;
	}
	UART_sends(buffer+stelle+1) ;
}

/* Receive on UART0 until the received byte is the specified char */
void UART_waitForChar(char ch)
{
	do
	{
		while ((U0LSR & (1 << 0))==0) ;
	} while ((U0RBR & 0xFF)!=ch) ;
}

/* wait some millisecs */
void Wait(unsigned long ms)
{
	unsigned long goal = ms + T1TC ;
	while (T1TC < goal) ;
}

/*! Initializes the external memory control
	a 1Mx32x4 SDRAM is connected (16MB)
	- original code by Steffen Graf -
*/
void init_ram_high(void){
  int i;

  PCONP|=(1<<11);
  //  EnableRAMPower();
  PINSEL5 = 0x55010115;
  PINMODE5 = 0xAAaaaaaA;
  PINSEL6 = 0x55555555;
  PINMODE6 = 0xAAAAAAAA;
  PINSEL7 = 0x55555555;
  PINMODE7 = 0xAAAAAAAA;
  PINSEL8 = 0x15555555;
  PINMODE8 = 0x1AAAAAAA;
  PINSEL9 = (1 << 18);
  PINMODE9 = 0x80000;
  
  EMC_CTRL=1; // enable EMC
  EMC_DYN_RD_CFG=1;
  EMC_DYN_RASCAS0|=0x303;
  EMC_DYN_RP= 4;//RP
  EMC_DYN_RAS = 8;//RAS
  EMC_DYN_SREX = 5;// SREX, XSR 
  EMC_DYN_APR = 2; //APR
  EMC_DYN_DAL = 5; //DAL, APW
  EMC_DYN_WR = 3; //WR, DPL, RWL, RDL
  EMC_DYN_RC = 11;
  EMC_DYN_RFC = 11; //rfc rc
  EMC_DYN_XSR = 5;//txsr
  EMC_DYN_RRD = 3;//RRD
  EMC_DYN_MRD = 8;//RAS
  EMC_DYN_CFG0 = (1<<14)|(1<<9)|(2<<7);

  
  Wait(1) ;
  EMC_DYN_CTRL = 0x0181; // NOP
  Wait(1) ;
  EMC_DYN_CTRL=0x101; 	 // PRECHARGE ALL
  Wait(1) ;
  EMC_DYN_CTRL=0x185;	 // request refresh
  Wait(1) ;
  EMC_DYN_CTRL=0x185;	 // request refresh
  Wait(1) ;
  

  EMC_DYN_CTRL=0x81; 	// MODE command
  Wait(1) ;
i = (*(volatile int*)(0xA0000000 | (0x32<<12)));
  Wait(1) ;
  EMC_DYN_CTRL = 1;		// normal SDRAM operation

  EMC_DYN_CFG0|=0x80000;  // buffer enable
}

int main(void){

	PLL_init() ;

	T1PR = FCPU / 1000 ; // count up every ms
	T1TCR = 1;
	
	InitUART() ;
	
	ALL_LED_ACTIVE();
	ALL_LED_OFF();
	
	UART_waitForChar('!') ;
	UART_sends("\r\n") ;
	UART_sends("\r\n") ;
	UART_sends("Hello World from the LPC2478\r\n") ;
	UART_sends("running at 80MHz Core, COM: 115200 baud 8,n,1\r\n") ;
	UART_sends("=============================================\r\n") ;
	UART_sends("Trying to init RAM: ") ;
	init_ram_high() ;
	UART_sends("Done\r\n") ;
	unsigned long i = 0, megs = 0 ;
	volatile unsigned char *ptr = (unsigned char *)0xA0000000 ;
	for (megs = 0;megs < 16; megs++)
	{
		int failed = 0 ;
		/* write */
		for (i=0;i<0x100000;i++)
		{
			unsigned char testval = i ^ (i >> 8) ^ (i >> 16) ^ (i >> 24) ;
			ptr[megs * 0x100000 + i] = (testval & 0xFF) ;
		}
		/* verify */
		for (i=0;i<0x100000;i++)
		{
			unsigned char testval = i ^ (i >> 8) ^ (i >> 16) ^ (i >> 24) ;
			if (ptr[megs * 0x100000 + i]!=testval)
			{
				UART_sends("Failed memory test at ") ;
				UART_sendx(megs * 0x100000 + i) ;
				UART_sends("\r\n") ;
				failed = 1 ;
				break ;
			}
		}
		if (failed) break ;
		UART_sends("Checked ") ;
		UART_sendu32(megs+1) ;
		UART_sends(" MB\r\n") ;
	}
	UART_sends("Blinking LEDS!\r\n") ;

	while(1){
		RED_LED_TOGGLE();
		Wait(100) ;
		BLUE_LED_TOGGLE();
		Wait(100) ;
		GREEN_LED_TOGGLE();
		Wait(100) ;
//		UART_sendc('.') ;
	}
}
